%% main.m
%% Autor: Julia I.M. Hauser
%% Datum: 01.03.2021
%% Lizenz: CC BY 4.0 International Julia I.M. Hauser. http://creativecommons.org/licenses/by/4.0/
%%
%% Beschreibung: Hier ist das zentrale Skript fuer die 1D FEM
%   Unter der Zeilen mit %%% sind Teile des Codes zu ergaenzen.

close all;
clear all;

%% Eingabe der Werte
f = @(x)-2.0*ones(size(x));
uorg = @(x) x.^2;

a = 0.0;
b = 1.0;
c = 0.0;
g_a = 0.0;
u_b = 3.0;
alpha_b = 1.0;
n = 4; %Anzahl der Intervalle, verändere die Anzahl um ein genaueres Ergebnis zu bekommen.

%% Berechnung der Daten
h = (b-a)/n;
interp_pkt = linspace(a,b,n+1);
intvals = zeros(n,2);
for i = 1:n
  intvals(i,:) = [ i, i+1 ];
end%for

%% Assemblierung der Gleichungssystems
[f_assemb, K_assemb] = assemblierung(f, interp_pkt, intvals);

%% Einbindung Randbedingungen
% Da wir Dirichletdaten als Anfangsbedingung haben, müssen wir die erste Zeile und Spalte streichen. Wegen der 
% Endbedingung müssen wir den letzten Eintrag der Matrix anpassen
%%%Ergänze die folgenden Zeilen
K_h = 
K_h(end,end) = 

f_h = f_assemb(2:end);
%Binde die Randbedingungen in die rechte Seite ein.
%%%Ergänze die folgenden Zeilen
f_h(1) = 
f_h(end) = 

%% Loese das Gleichungssystem und setze die Anfangsdaten
uh = zeros(n+1,1);
uh(1) = g_a;
uh(2:end) = K_h\f_h;

%% Plotte Loesung
plot_punkte = linspace(a,b,10*n+1);
uh_plot= zeros(size(plot_punkte));
for j = 1:n
  xim1 = interp_pkt(j);
  xi = interp_pkt(j+1);
  % Wenn das gesamte Intervall [xim1,xi] genommen wird muss bei allen inneren 
  % Stützpunkten die Werte halbiert werden, da sie zwei Mal dazugezaehlt wurden.
  check = (plot_punkte>=xim1) & (plot_punkte<xi);
  points = plot_punkte(check);
  uh_plot(check) = uh_plot(check) +uh(j)* phii(0, points, xim1,xi);
  uh_plot(check) = uh_plot(check) +uh(j+1)* phii(1, points, xim1,xi);
end%for
uh_plot(end) = uh_plot(end) + uh(end)* phii(1, plot_punkte(end),...
                             interp_pkt(end-1),interp_pkt(end));
                             
% Berechne den punktweisen Fehler in den Plotpunkten
error = sum(( uorg(plot_punkte) -  uh_plot).^2)                            
   
% Erstelle den Plot   
figure();
plot(plot_punkte, uorg(plot_punkte), 'color', 'red');
hold on;
plot(plot_punkte, uh_plot,'color', 'blue');    
hleg = legend ("u_{org}", "u_h");
legend (hleg, "location", "northeastoutside");
set (hleg, "fontsize", 14);
plot(interp_pkt, zeros(size(interp_pkt)),'o');
title ("Loesung der Gleichung -u''(x) = -2");
xlabel("Interpolationspunkte");
ylabel("u_{org}(x) bzw. u_h(x)");   
hold off;